addHook("NetVars", function(net)
	chaosmod.permeffect = net($)
end)

rawset(_G, "ckAddPermEffect", function(funcs)
	local tablelength = #ckPermEffects
	if funcs.rare == true
		table.insert(ckRarePermEffects, funcs)
	else
		table.insert(ckPermEffects, funcs)
	end
	if gametype == GT_BATTLE and funcs.battlemode ~= false
		if funcs.rare == true
			table.insert(ckBattleRarePermEffects, funcs)
		else
			table.insert(ckBattlePermEffects, funcs)
		end
	end
end)

// Allows Perminant Effects to be called
rawset(_G, "cv_permeffects", CV_RegisterVar({
  name = "ck_permeffects",
  defaultvalue = "On",
  flags = CV_NETVAR,
  PossibleValue = {Off = 0, On = 1},
}))

// Always calls a Permanent Effect
rawset(_G, "cv_alwayspermeffects", CV_RegisterVar({
  name = "ck_alwayspermeffect",
  defaultvalue = "Off",
  flags = CV_NETVAR,
  PossibleValue = {Off = 0, On = 1},
}))

// Always calls a Rare Permanent Effect
rawset(_G, "cv_alwaysrarepermeffects", CV_RegisterVar({
  name = "ck_alwaysrarepermeffect",
  defaultvalue = "Off",
  flags = CV_NETVAR,
  PossibleValue = {Off = 0, On = 1},
}))

// 10% Chance for a perminant effect to run every level.
addHook("MapLoad", function()
	if cv_chaoskart.value == 1
		if P_RandomChance(FRACUNIT/10)
		or cv_alwayspermeffects.value == 1
			local rarechance = ckEventChance()
			if rarechance <= 90 and cv_alwaysrarepermeffects.value == 0
				if gametype == GT_BATTLE
					ckBattlePermEffect(P_RandomRange(1, #ckBattlePermEffects))
				else
					ckPermEffect(P_RandomRange(1, #ckPermEffects))
				end
			else
				if gametype == GT_BATTLE
					ckBattleRarePermEffect(P_RandomRange(1, #ckBattleRarePermEffects))
				else
					ckRarePermEffect(P_RandomRange(1, #ckRarePermEffects))
				end
			end
		else
			chaosmod.permeffect = nil
		end
	end
end)

// Calls a perm effect.
rawset(_G, "ckPermEffect", function(effect)
	chaosmod.permeffect = ckPermEffects[effect].name
	if encoremode == true and ckPermEffects[effect].encoredesc
		chaosmod.permeffectdesc = ckPermEffects[effect].encoredesc
	else
		chaosmod.permeffectdesc = ckPermEffects[effect].desc
	end
	if ckPermEffects[effect].starteffect ~= nil
		ckPermEffects[effect].starteffect()
	end
	S_StartSound(nil, ckPermEffects[effect].startsound)
end)

// Calls a rare perm effect.
rawset(_G, "ckRarePermEffect", function(effect)
	chaosmod.permeffect = ckRarePermEffects[effect].name
	if encoremode == true and ckRarePermEffects[effect].encoredesc
		chaosmod.permeffectdesc = ckRarePermEffects[effect].encoredesc
	else
		chaosmod.permeffectdesc = ckRarePermEffects[effect].desc
	end
	if ckRarePermEffects[effect].starteffect ~= nil
		ckRarePermEffects[effect].starteffect()
	end
	S_StartSound(nil, ckRarePermEffects[effect].startsound)
end)

// Calls a battle only perm effect.
rawset(_G, "ckBattlePermEffect", function(effect)
	chaosmod.permeffect = ckBattlePermEffects[effect].name
	if encoremode == true and ckBattlePermEffects[effect].encoredesc
		chaosmod.permeffectdesc = ckBattlePermEffects[effect].encoredesc
	else
		chaosmod.permeffectdesc = ckBattlePermEffects[effect].desc
	end
	if ckBattlePermEffects[effect].starteffect ~= nil
		ckBattlePermEffects[effect].starteffect()
	end
	S_StartSound(nil, ckBattlePermEffects[effect].startsound)
end)

// Calls a battle only rare perm effect
rawset(_G, "ckBattleRarePermEffect", function(effect)
	chaosmod.permeffect = ckBattleRarePermEffects[effect].name
	if encoremode == true and ckBattleRarePermEffects[effect].encoredesc
		chaosmod.permeffectdesc = ckBattleRarePermEffects[effect].encoredesc
	else
		chaosmod.permeffectdesc = ckBattleRarePermEffects[effect].desc
	end
	if ckBattleRarePermEffects[effect].starteffect ~= nil
		ckBattleRarePermEffects[effect].starteffect()
	end
	S_StartSound(nil, ckBattleRarePermEffects[effect].startsound)
end)

addHook("PlayerJoin", function(p)
	for i in pairs(ckPermEffects)
		if chaosmod.permeffect == ckPermEffects[i].name
			if (ckPermEffects[i].starteffect ~= nil and ckPermEffects[i].synchnewjoiners == true)
				ckPermEffects[i].starteffect()
			end
		end
	end

	for i in pairs(ckRarePermEffects)		
		if chaosmod.permeffect == ckRarePermEffects[i].name
			if (ckRarePermEffects[i].starteffect ~= nil and ckRarePermEffects[i].synchnewjoiners == true)
				ckRarePermEffects[i].starteffect()
			end
		end
	end
end)

// Perm effect execution.
addHook("ThinkFrame", function()
	if leveltime >= 6*TICRATE
		for i in pairs(ckPermEffects)
			if encoremode == true
				if (chaosmod.permeffect == ckPermEffects[i].name) and (ckPermEffects[i].encoreeffect) then ckPermEffects[i].encoreeffect() end
			else
				if (chaosmod.permeffect == ckPermEffects[i].name) and (ckPermEffects[i].effect) then ckPermEffects[i].effect() end
			end
		end
		
		for i in pairs(ckRarePermEffects)
			if encoremode == true
				if (chaosmod.permeffect == ckRarePermEffects[i].name) and (ckRarePermEffects[i].encoreeffect) then ckRarePermEffects[i].encoreeffect() end
			else
				if (chaosmod.permeffect == ckRarePermEffects[i].name) and (ckRarePermEffects[i].effect) then ckRarePermEffects[i].effect() end
			end
		end
	end
end)

// Perm effect mobjthinkers.
addHook("MobjThinker", function(mo)
	if leveltime >= 6*TICRATE
		for i in pairs(ckPermEffects)
			if chaosmod.permeffect == ckPermEffects[i].name
				if ckPermEffects[i].mobjthinker then ckPermEffects[i].mobjthinker(mo) end
			end
		end
		
		for i in pairs(ckRarePermEffects)
			if chaosmod.permeffect == ckRarePermEffects[i].name
				if ckRarePermEffects[i].mobjthinker then ckRarePermEffects[i].mobjthinker(mo) end
			end
		end
	end
end)

// Perm effect spineffects
addHook("PlayerSpin", function(p, i, s)
	for m in pairs(ckPermEffects)
		if chaosmod.permeffect == ckPermEffects[m].name
			if ckPermEffects[m].spineffect then ckPermEffects[m].spineffect(p, i, s) end
		end
	end
	
	for m in pairs(ckRarePermEffects)
		if chaosmod.permeffect == ckRarePermEffects[m].name
			if ckRarePermEffects[m].spineffect then ckRarePermEffects[m].spineffect(p, i, s) end
		end
	end
end)

// Perm effect xpldeffects
addHook("PlayerExplode", function(p, i, s)
	for m in pairs(ckPermEffects)
		if chaosmod.permeffect == ckPermEffects[m].name
			if ckPermEffects[m].xpldeffect then ckPermEffects[m].xpldeffect(p, i, s) end
		end
	end
		
	for m in pairs(ckRarePermEffects)
		if chaosmod.permeffect == ckRarePermEffects[m].name
			if ckRarePermEffects[m].xpldeffect then ckRarePermEffects[m].xpldeffect(p, i, s) end
		end
	end
end)

// Perm effect HUD.
hud.add(function(v, p, cam)
	if cv_chaoskart.value == 1
		if chaosmod.permeffect ~= nil and leveltime <= 7*TICRATE
			v.drawKartString(160, 30 ,chaosmod.permeffect, V_SNAPTOLEFT|V_SNAPTOTOP)
			v.drawString(160, 52, chaosmod.permeffectdesc, V_SNAPTOLEFT|V_SNAPTOTOP, "small")
		end
	end
end)